import datetime
import functools
import json
import operator
from collections import Counter

import redis

import pymysql
from pyspark import SparkConf
from pyspark.sql import SparkSession
from tool import (get_doris_prod, get_jerry_test, get_tag3_user_log, send_email, write_user_portrait,
                  write_user_portrait_by_event, write_user_portrait_doris)


# [{'激光': 1.949194898204873}, {'手术': 1.949194898204873}, {'手术': 1.949194898204873}, {'手术': 1.949194898204873, '植发际线': 7.1}]
# {'手术': 5.8475846946146195, '激光': 1.949194898204873}
def merge_values(list_of_dict):
    d = dict(functools.reduce(operator.add, map(Counter, list_of_dict)))
    return dict(sorted(d.items(), key=lambda x: x[1], reverse=True))


# [("a", 1), ("b", 2)]
# {'a': 1, 'b': 2}
# [{'缩阴道激光': 1.949194898204873}, {'': 1.949194898204873}, {'': 1.949194898204873}, {'私处手术': 1.949194898204873}]
def make_dict_from_pair(x):
    if "" in x[0]:
        x[0].remove("")
    return dict(zip(x[0], [x[1]] * len(x[0])))


# [('点击内容卡片', {'假体隆胸': 1.9067764699346477}), ('点击内容卡片', {'植发': 1.9067764699346477, '植发际线': 1})]
# {'搜索操作': {'手术': 43.8116976521965, '齿科治疗': 26.63919566939707}, '主动咨询': {'消脂': 12.612304719468522}}
def merge_results_by_event(lst):
    tmp = {}
    for i in lst:
        event_cn = i[0]
        d = i[1]
        if d:
            if event_cn not in tmp:
                tmp[event_cn] = [d]
            else:
                tmp[event_cn].append(d)
    res = {}
    for (k, v) in tmp.items():
        d2 = merge_values(v)
        lst = sorted(d2.items(), key=lambda x: x[1], reverse=True)
        tmp_lst = []
        for i in lst:
            tmp_lst.append(i[0])
        if tmp_lst:
            res[k] = tmp_lst
    return res


def _make_score(lst, score):
    return dict(zip(lst, [score] * len(lst)))


def _get_all_by_project(name):
    first_demands = []
    second_demands = []
    first_solutions = []
    second_solutions = []
    first_positions = []
    second_positions = []
    sql = """select name, tag_type from api_tag_3_0 where id in (
             select tag_attr.tag_attr_id from api_tag_3_0 as api_tag
             left join api_tag_attr_tag as tag_attr on tag_attr.tag_id = api_tag.id
             where api_tag.name = '{}')
    """.format(name)
    zhengxing_conn = pymysql.connect(host="172.16.30.141",
                                     user="work",
                                     password="BJQaT9VzDcuPBqkd",
                                     database="zhengxing",
                                     charset="utf8")
    zhengxing_cursor = zhengxing_conn.cursor()
    zhengxing_cursor.execute(sql)
    sql_tag_results = zhengxing_cursor.fetchall()
    # TAG_V3_TYPE
    for name, agg_type in sql_tag_results:
        if agg_type == 20:
            second_demands.append(name)
        elif agg_type == 16:
            second_solutions.append(name)
        elif agg_type == 22:
            second_positions.append(name)
        elif agg_type == 19:
            first_demands.append(name)
        elif agg_type == 18:
            first_solutions.append(name)
        elif agg_type == 21:
            first_positions.append(name)
    return {
        "first_demands": first_demands,
        "second_demands": second_demands,
        "first_solutions": first_solutions,
        "second_solutions": second_solutions,
        "first_positions": first_positions,
        "second_positions": second_positions,
        "projects": [name]
    }


def _get_all_by_projects(name_lst):
    first_demands = []
    second_demands = []
    first_solutions = []
    second_solutions = []
    first_positions = []
    second_positions = []
    projects = name_lst
    for name in projects:
        all = _get_all_by_project(name)
        first_demands.extend(all.get("first_demands", []))
        second_demands.extend(all.get("second_demands", []))
        first_solutions.extend(all.get("first_solutions", []))
        second_solutions.extend(all.get("second_solutions", []))
        first_positions.extend(all.get("first_positions", []))
        second_positions.extend(all.get("second_positions", []))
    if first_demands or second_demands or first_solutions or second_solutions or first_positions or second_positions or projects:
        return {
            "first_demands": _make_score(first_demands, 2),
            "second_demands": _make_score(second_demands, 2),
            "first_solutions": _make_score(first_solutions, 2),
            "second_solutions": _make_score(second_solutions, 2),
            "first_positions": _make_score(first_positions, 2),
            "second_positions": _make_score(second_positions, 2),
            "projects": _make_score(projects, 2)
        }
    return None


def update_tag3_user_portrait_by_event(cl_id):
    user_df = get_tag3_user_log(cl_id)
    if not user_df.empty:
        user_df["first_solutions"] = list(zip(user_df["first_solutions"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["second_solutions"] = list(zip(user_df["second_solutions"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["first_demands"] = list(zip(user_df["first_demands"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["second_demands"] = list(zip(user_df["second_demands"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["first_positions"] = list(zip(user_df["first_positions"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["second_positions"] = list(zip(user_df["second_positions"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["projects"] = list(zip(user_df["projects"].apply(lambda x: x.split(",")), user_df["tag_score"]))

        user_df["first_solutions_dict"] = user_df["first_solutions"].apply(lambda x: make_dict_from_pair(x))
        user_df["second_solutions_dict"] = user_df["second_solutions"].apply(lambda x: make_dict_from_pair(x))
        user_df["first_demands_dict"] = user_df["first_demands"].apply(lambda x: make_dict_from_pair(x))
        user_df["second_demands_dict"] = user_df["second_demands"].apply(lambda x: make_dict_from_pair(x))
        user_df["first_positions_dict"] = user_df["first_positions"].apply(lambda x: make_dict_from_pair(x))
        user_df["second_positions_dict"] = user_df["second_positions"].apply(lambda x: make_dict_from_pair(x))
        user_df["projects_dict"] = user_df["projects"].apply(lambda x: make_dict_from_pair(x))

        user_df["first_solutions"] = list(zip(user_df["event_cn"], user_df["first_solutions_dict"]))
        user_df["second_solutions"] = list(zip(user_df["event_cn"], user_df["second_solutions_dict"]))
        user_df["first_demands"] = list(zip(user_df["event_cn"], user_df["first_demands_dict"]))
        user_df["second_demands"] = list(zip(user_df["event_cn"], user_df["second_demands_dict"]))
        user_df["first_positions"] = list(zip(user_df["event_cn"], user_df["first_positions_dict"]))
        user_df["second_positions"] = list(zip(user_df["event_cn"], user_df["second_positions_dict"]))
        user_df["projects"] = list(zip(user_df["event_cn"], user_df["projects_dict"]))

        first_solutions_dict = merge_results_by_event(user_df["first_solutions"].tolist())
        second_solutions_dict = merge_results_by_event(user_df["second_solutions"].tolist())
        first_demands_dict = merge_results_by_event(user_df["first_demands"].tolist())
        second_demands_dict = merge_results_by_event(user_df["second_demands"].tolist())
        first_positions_dict = merge_results_by_event(user_df["first_positions"].tolist())
        second_positions_dict = merge_results_by_event(user_df["second_positions"].tolist())
        projects_dict = merge_results_by_event(user_df["projects"].tolist())

        events = set(
            list(first_solutions_dict.keys()) + list(second_solutions_dict.keys()) + list(first_demands_dict.keys()) +
            list(second_demands_dict.keys()) + list(first_positions_dict.keys()) + list(second_positions_dict.keys()) +
            list(projects_dict.keys()))

        for e in events:
            first_solutions = ",".join(first_solutions_dict.get(e, []))
            second_solutions = ",".join(second_solutions_dict.get(e, []))
            first_demands = ",".join(first_demands_dict.get(e, []))
            second_demands = ",".join(second_demands_dict.get(e, []))
            first_positions = ",".join(first_positions_dict.get(e, []))
            second_positions = ",".join(second_positions_dict.get(e, []))
            projects = ",".join(projects_dict.get(e, []))

            write_user_portrait_by_event(cl_id, first_solutions, second_solutions, first_demands, second_demands, first_positions,
                                         second_positions, projects, e)

        return cl_id


def update_tag3_user_portrait(cl_id):
    user_df = get_tag3_user_log(cl_id)
    if not user_df.empty:
        user_df["first_solutions"] = list(zip(user_df["first_solutions"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["second_solutions"] = list(zip(user_df["second_solutions"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["first_demands"] = list(zip(user_df["first_demands"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["second_demands"] = list(zip(user_df["second_demands"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["first_positions"] = list(zip(user_df["first_positions"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["second_positions"] = list(zip(user_df["second_positions"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["projects"] = list(zip(user_df["projects"].apply(lambda x: x.split(",")), user_df["tag_score"]))

        user_df["first_solutions_dict"] = user_df["first_solutions"].apply(lambda x: make_dict_from_pair(x))
        user_df["second_solutions_dict"] = user_df["second_solutions"].apply(lambda x: make_dict_from_pair(x))
        user_df["first_demands_dict"] = user_df["first_demands"].apply(lambda x: make_dict_from_pair(x))
        user_df["second_demands_dict"] = user_df["second_demands"].apply(lambda x: make_dict_from_pair(x))
        user_df["first_positions_dict"] = user_df["first_positions"].apply(lambda x: make_dict_from_pair(x))
        user_df["second_positions_dict"] = user_df["second_positions"].apply(lambda x: make_dict_from_pair(x))
        user_df["projects_dict"] = user_df["projects"].apply(lambda x: make_dict_from_pair(x))

        first_solutions_list = user_df["first_solutions_dict"].tolist()
        second_solutions_list = user_df["second_solutions_dict"].tolist()
        first_demands_list = user_df["first_demands_dict"].tolist()
        second_demands_list = user_df["second_demands_dict"].tolist()
        first_positions_list = user_df["first_positions_dict"].tolist()
        second_positions_list = user_df["second_positions_dict"].tolist()
        projects_list = user_df["projects_dict"].tolist()

        first_demands_score = merge_values(first_demands_list)
        second_demands_score = merge_values(second_demands_list)
        first_solutions_score = merge_values(first_solutions_list)
        second_solutions_score = merge_values(second_solutions_list)
        first_positions_score = merge_values(first_positions_list)
        second_positions_score = merge_values(second_positions_list)
        projects_score_tmp = merge_values(projects_list)

        tmp = [
            "不感兴趣", "没有想法", "八卦来了", "颜值速报", "医美审美干货", "其他项目", "网红颜值", "少年之名", "郑人予", "热点课代表", "私密", "陈瑞泽", "符仁杰", "祖力亚尔", "刘泽旭",
            "陈家瑨", "娱乐八卦", "美白1", "情感", "颜安", "苏勋伦", "丰唇1", "牙齿矫正", "无创溶脂"
        ]
        forbidden_lst = ["无创溶脂kyc"]
        for i in tmp:
            forbidden_lst.append(i + "kyc")
        forbidden_lst.extend(tmp)
        forbidden_set = set(forbidden_lst)

        projects_score = {}
        for (k, v) in projects_score_tmp.items():
            if k not in forbidden_set:
                projects_score[k] = v

        tmp_res = {}
        if (len(first_demands_score) == 0) and (len(first_solutions_score) == 0) and (len(first_positions_score) == 0) and (
                len(second_demands_score) == 0) and (len(second_solutions_score) == 0) and (len(second_positions_score) == 0):
            tmp_res = _get_all_by_projects(list(projects_score.keys()))

        res = {
            "first_demands": first_demands_score,
            "second_demands": second_demands_score,
            "first_solutions": first_solutions_score,
            "second_solutions": second_solutions_score,
            "first_positions": first_positions_score,
            "second_positions": second_positions_score,
            "projects": projects_score
        }
        if tmp_res:
            res.update(tmp_res)

        key = "doris:user_portrait:tag3:device_id:" + str(cl_id)
        redis_client = redis.StrictRedis.from_url("redis://:ReDis!GmTx*0aN9@172.16.40.173:6379")

        if (len(first_demands_score.keys()) > 0) or (len(second_demands_score.keys()) > 0) or \
           (len(first_solutions_score.keys()) > 0) or (len(second_solutions_score.keys()) > 0) or \
           (len(first_positions_score.keys()) > 0) or (len(second_positions_score.keys()) > 0) or \
           (len(projects_score.keys()) > 0):
            redis_client.set(key, json.dumps(res))
            redis_client.expire(key, 60 * 60 * 24 * 30)

            write_user_portrait(cl_id, ",".join(first_solutions_score.keys()), ",".join(second_solutions_score.keys()),
                                ",".join(first_demands_score.keys()), ",".join(second_demands_score.keys()),
                                ",".join(first_positions_score.keys()), ",".join(second_positions_score.keys()),
                                ",".join(projects_score.keys()))

            # write_user_portrait_doris(cl_id, ",".join(first_solutions_score.keys()), ",".join(second_solutions_score.keys()),
            #                           ",".join(first_demands_score.keys()), ",".join(second_demands_score.keys()),
            #                           ",".join(first_positions_score.keys()), ",".join(second_positions_score.keys()),
            #                           ",".join(projects_score.keys()))

        return cl_id


def consume_kafka():
    sql = "select distinct cl_id from kafka_tag3_log where log_time > UNIX_TIMESTAMP(DATE_SUB(NOW(), INTERVAL 30 day))"
    db, cursor = get_doris_prod()
    cursor.execute(sql)
    device_ids_lst = [i[0] for i in cursor.fetchall()]
    db.close()
    cursor.close()

    sparkConf = SparkConf().set("spark.hive.mapred.supports.subdirectories", "true") \
                           .set("spark.hadoop.mapreduce.input.fileinputformat.input.dir.recursive", "true") \
                           .set("spark.tispark.plan.allow_index_double_read", "false") \
                           .set("spark.tispark.plan.allow_index_read", "true") \
                           .set("spark.sql.extensions", "org.apache.spark.sql.TiExtensions") \
                           .set("spark.tispark.pd.addresses", "172.16.40.170:2379") \
                           .set("spark.io.compression.codec", "lzf") \
                           .set("spark.driver.maxResultSize", "8g") \
                           .set("spark.sql.avro.compression.codec", "snappy")

    try:
        spark = SparkSession.builder.config(conf=sparkConf).enableHiveSupport().getOrCreate()
        spark.sparkContext.setLogLevel("WARN")
        spark.sparkContext.addPyFile("/srv/apps/ffm-baseline_git/eda/smart_rank/tool.py")

        device_ids_lst_rdd = spark.sparkContext.parallelize(device_ids_lst, numSlices=1000)

        result = device_ids_lst_rdd.repartition(100).map(lambda x: update_tag3_user_portrait(x))
        # result.foreach(print)
        result.collect()

        result2 = device_ids_lst_rdd.repartition(100).map(lambda x: update_tag3_user_portrait_by_event(x))
        # result2.foreach(print)
        result2.collect()

        spark.stop()
    except Exception as e:
        send_email("tag3_update_user_portrait_offline", "tag3_update_user_portrait_offline", e)


if __name__ == "__main__":
    start = datetime.datetime.now()

    # update_tag3_user_portrait("androidid_a25a1129c0b38f7b")
    # device_id = "862460044588666"
    # update_tag3_user_portrait(device_id)

    consume_kafka()
    end = datetime.datetime.now()
    print(end - start)
    print("done")
